@@ -15,10 +15,6 @@
 
 #include "compat-autoconf.h"
 
-#if LINUX_VERSION_IS_LESS(5, 10, 0)
-#include "net/genetlink.h"
-#endif /* LINUX_VERSION_IS_LESS(5, 10, 0) */
-
 #endif /* __KERNEL__ */
 
 #endif /* _NET_BATMAN_ADV_COMPAT_H_ */
